#!/usr/bin/python
# -*- coding: cp936 -*-
import serial
import sys
import visa
import time
import numpy as np
import binascii
import matplotlib.pyplot as plt
from math import *

log_name = "6675A_VHP100_40Kx4p_SR104_0505_TCR.csv"

temp_offset = 2.7

rm = visa.ResourceManager()
#dmm = rm.open_resource("GPIB0::11::INSTR") #reference DMM HP34401A
dmm = rm.open_resource("GPIB0::22::INSTR") #reference DMM HP3458A#1
G6675 = rm.open_resource("GPIB1::1::INSTR") #6675

#dmm.write("*RST")
#time.sleep(5)
#dmm.write("*CLS")
#dmm.write("CONF:FRES 1E4, MAX")
#dmm.write("SENS:ZERO:AUTO ON")
#dmm.write("SENS:FRES:RES MAX")
#dmm.write("SENS:FRES:NPLC 100")

dmm.write("RESET")
dmm.write("END ALWAYS")
dmm.write("OHMF 10E3")
dmm.write("NPLC 100")
dmm.write("NDIG 8")
dmm.write("TRIG AUTO")
dmm.write("INBUF ON")
dmm.timeout = 300000

#dmm.timeout = 30000    
G6675.timeout = 60000
print "6675A configured."



def recv(serial):
    serial.flushInput()
    rx = ''
    while True:
        num = serial.inWaiting()
        rx_raw = serial.read(num)
        if rx_raw and len(rx_raw)>0:
            rx_raw = ''.join(rx_raw)
            rx += binascii.hexlify(bytearray(rx_raw))
            index = rx.find('5a5a15')        
            if (index != -1) and (len(rx)-index) == 48:
                readbuf = rx[index:]
                break
            else:
                continue        
    return readbuf

def crc_sum(data):
    crc_data = 0
    for i in range(0,8):
        crc_data += int(binascii.hexlify(data[i]),16)
        i += 1
    if (int(binascii.hexlify(data[8]),16) == crc_data & 0x000000FF):
        crc_data = 0
        for i in range(0,14):
            crc_data += int(binascii.hexlify(data[i+9]),16)
            i += 1
        if (int(binascii.hexlify(data[23]),16) == crc_data & 0x000000FF):
            return 1
        else:
            return -1
    else:
        return -1
    
def ambient_status():
    amb = []
    data = recv(serial)
    data = data.decode('hex')
    
    if (crc_sum(data) != -1):
#        data = data.decode('hex')
        light = int(binascii.hexlify(data[4:8]),16)/100.0   #light
        temp = int(binascii.hexlify(data[13:15]),16)/100.0 #temp
        pressure = int(binascii.hexlify(data[15:19]),16)/100.0 #pressure
        huminty = int(binascii.hexlify(data[19:21]),16)/100.0 #huminty
        high = int(binascii.hexlify(data[21:23]),16)/100.0 #high
        amb = [light,temp,pressure,huminty,high]
    else:
        print('crc error')
    return amb       

if __name__ == '__main__':
    serial = serial.Serial('COM6', 9600, timeout=1)
    if serial.isOpen():
        print("Serial port open success\r\n")
    else:
        print("Serial port open failed\r\n")
        
    with open(log_name, 'a') as dile:
        dile.write ("date,Rx,Rt,Temp,Temp_amb,Huminty,Pressure,high,light\r\n")
#        dile.write ("date,Rx,Rt,Temp,Temp_amb,Huminty,Pressure,high,light\r\n")
    dile.close()
    

    vcnt = 0 # temporary counter
    print "Log into %s started" % log_name

    run = 0
    val = 0
    val1 = 0
    temp = 0
    data = []
    array = []



    while run < 50001:
        array = []
        for ix in range (0,7): #take more samples
            #val = float(dmm.query("READ?"))#34401
            dmm.write("TARM SGL")
            val1 = float(dmm.read())
            array.extend([val1])
        val1 = np.median(array[1:])
        temp = 23.0 + (val1 - 10000)/(0.001*10000)
        
        amb = ambient_status()
        
        time.sleep(1)
        G6675.write("FETC?")
        val = float(G6675.read())
        
            
        with open(log_name, 'a') as dile:
#            dile.write (time.strftime("%Y-%m-%d %H:%M:%S, ") + ("%.9f,%.6f,%.2f,%.2f,%.5f,%.2f,%.2f\r\n" %(val, temp, amb[1], amb[3], amb[2], amb[4], amb[0])))
            dile.write (time.strftime("%Y-%m-%d %H:%M:%S, ") + ("%.9f,%.9f,%.2f,%.2f,%.2f,%.5f,%.2f,%.2f\r\n" %(val, val1, temp, amb[1]-temp_offset, amb[3], amb[2], amb[4], amb[0])))
        dile.close()
        print("run:%4d, g6675A: %.6f, %.6f, %.3f, %.3f, %.2f%%, %.5fkPa" %(run, val, val1, temp, amb[1]-temp_offset, amb[3], amb[2]/1000.0))
 
        run = run + 1
        
